
// FILE: src/GridNavigationLibrary.js
function GridNavigationLibrary(ZhKeyboardService, templateManager, ScreenService, Query) {

	var rowHeight = 35;
	var footerHeigth = 42;

	var isTree = function (widget) {
		return widget.template == "/widget/tree/grid.html" || widget.template == "widget/tree/grid.html";
	};

	var getWidgetFromAngularScope = function () {
		var container = $('.popup-content');
		if (container.length === 0) {
			container = $('.zh-swipe-container').find('.zh-swipe-inner').last();
			if (container.length === 0) {
				container = $('.everything');
			}
		}
		var element = angular.element(container.find('.zh-grid-container').last());

		if (element.scope()) {
			return element.scope().widget;
		}
	};

	function _getFocusedElement() {
		var element = $(':focus');
		if (element.is('.zh-label-with-icon') ||
			element.parent().is('.zh-radio-label') ||
			element.parent().is('.zh-box-label') ||
			element.is('textarea') ||
			element.is('input')) {
			return element;
		} else {
			return undefined;
		}
	}

	templateManager.projectPromise.then(function() {
		var keys = {
			toogleRow : templateManager.project.shortcutKeys.toogleRow || "space",
			openSwipe : templateManager.project.shortcutKeys.openSwipe || "enter",
			openInlineEdit : templateManager.project.shortcutKeys.openInlineEdit || "ctrl+space"
		};

		ZhKeyboardService.bind(keys.toogleRow, function () {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					if (widget.showCheckbox) {
						widget.toggle(widget.currentRow);
					}
					templateManager.updateTemplate();
				}
			}
		});

		ZhKeyboardService.bind(keys.openSwipe, function () {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					if (widget.name == "select_multiple_widget") {
						widget.setSelected(widget.dataSource.getCheckedRows(), widget.selectField);
						widget.afterClosePopup(widget.selectField);
					} else if (widget.afterMoveRow) {
						widget.afterMoveRow();
					}
				}
			}
		});

		ZhKeyboardService.bind(keys.openInlineEdit, function(event) {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				var elem = $($('.currentRow').find('.inline-editable')[0]);
				event.preventDefault();

				if(widget.template.indexOf('grid') >= 0 && elem.attr('class') && elem.attr('class').indexOf('inline-editable') >= 0){
					elem.click();
				}
			}
		});

		ZhKeyboardService.bind("right", function () {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					if (isTree(widget) && !Util.isEmptyObject(widget.currentRow)) {
						widget.currentRow._opened = true;
						if (widget.resizeGridColumns) {
							widget.resizeGridColumns(true);
						}
						templateManager.updateTemplate();
					} else if (widget.currentRow.__group && widget.currentRow.__groupHeader) {
						widget.currentRow.__groupOpened = true;
						templateManager.updateTemplate();
					}
				}
			}
		});

		ZhKeyboardService.bind("left", function () {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					if (isTree(widget) && !Util.isEmptyObject(widget.currentRow)) {
						widget.currentRow._opened = false;
						templateManager.updateTemplate();
					} else if (widget.currentRow.__group && widget.currentRow.__groupHeader) {
						widget.currentRow.__groupOpened = false;
						templateManager.updateTemplate();
					}
				}
			}
		});

		ZhKeyboardService.bind('pageup', function (event) {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					event.preventDefault();

					var parent = $('.body').last();
					var widgetHeight = parent.height();
					var leftover = parent.height() % rowHeight;
					if (parent.siblings('.grid-vertical-scroll-control').length > 0){
						parent.siblings('.grid-vertical-scroll-control').scrollTop(parent.siblings('.grid-vertical-scroll-control').scrollTop() - widgetHeight + leftover);
					} else {
						parent.scrollTop(parent.scrollTop() - widgetHeight + leftover);
					}
				}
			}
		});

		ZhKeyboardService.bind('pagedown', function (event) {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					event.preventDefault();

					var parent = $('.body').last();
					var widgetHeight = parent.height();
					var leftover = parent.height() % rowHeight;
					if (parent.siblings('.grid-vertical-scroll-control').length > 0){
						parent.siblings('.grid-vertical-scroll-control').scrollTop(parent.siblings('.grid-vertical-scroll-control').scrollTop() + widgetHeight - leftover);
					} else {
						parent.scrollTop(parent.scrollTop() + widgetHeight - leftover);
					}
				}
			}
		});

		ZhKeyboardService.bind("up", function (event) {
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var dataSource;
				var widget = getWidgetFromAngularScope();
				if (widget) {
					event.preventDefault();
					if (isTree(widget) && !!widget.searchList) {
						dataSource = filterSearchVisibleRows(widget, widget.plainTreeData);
					} else if (isTree(widget) && !widget.searchList) {
						dataSource = filterVisibleRowsOnTree(widget);
					} else if (!isTree(widget) && !!widget.searchList) {
						dataSource = filterSearchVisibleRows(widget, widget.groupedData || widget.dataSource.data);
					} else if (!isTree(widget) && !widget.searchList) {
						dataSource = widget.groupedData || widget.dataSource.data;
					}
					pressUpKey(widget, dataSource);
					templateManager.updateTemplate();
					var parent = $('.body').last();
					var scrollControl = parent.siblings('.grid-vertical-scroll-control')[0];
					var row = parent.find(".currentRow:last");

					if (scrollControl) {
						if (row.length === 0 || row[0] == parent.find('.tr').first()[0]) {
							scrollControl.scrollTop -= rowHeight;
						}
					} else {
						if (row.length > 0 && row.offset().top < parent.offset().top) {
							parent[0].scrollTop -= rowHeight;
						}
					}
				}
			}
		});

		ZhKeyboardService.bind("down", function (event) {
			var dataSource;
			if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
				var widget = getWidgetFromAngularScope();
				if (widget) {
					event.preventDefault();
					if (isTree(widget) && !!widget.searchList) {
						dataSource = filterSearchVisibleRows(widget, widget.plainTreeData);
					} else if (isTree(widget) && !widget.searchList) {
						dataSource = filterVisibleRowsOnTree(widget);
					} else if (!isTree(widget) && !!widget.searchList) {
						dataSource = filterSearchVisibleRows(widget, widget.groupedData || widget.dataSource.data);
					} else if (!isTree(widget) && !widget.searchList) {
						dataSource = widget.groupedData || widget.dataSource.data;
					}
					pressDownKey(widget, dataSource);
					templateManager.updateTemplate();
					var parent = $('.body').last();
					var scrollControl = parent.siblings('.grid-vertical-scroll-control')[0];
					var row = parent.find(".currentRow:last");

					if (scrollControl) {
						if (row.length === 0 || row[0] == parent.find('.tr').last()[0]) {
							scrollControl.scrollTop += rowHeight;
						}
					} else {
						if (row.length > 0 && row.offset().top + rowHeight > parent.offset().top + parent.innerHeight()) {
							parent[0].scrollTop += rowHeight;
						}
					}
				}
			}
		});
	});

	var pressUpKey = function (widget, dataSource) {
		var previousCurrentRow = widget.currentRow;
		var index = dataSource.indexOf(widget.currentRow);
		if (Util.isEmptyObject(widget.currentRow) || index == -1) {
			widget.currentRow = dataSource[0];
		} else if (index > 0) {
			var isClosed, row, found = false;
			do {
				index -= 1;
				row = dataSource[index];
				isClosed = row.__groupHeaders && row.__groupHeaders.some(function(header) {
					return header.__groupOpened === false;
				});
				found = !isClosed;
			} while (!found && index > 0);
			if (found) {
				widget.currentRow = dataSource[index];
			}
		}

		var element = angular.element($('.body').last());
		if (element.scope() && element.scope().controller && element.scope().controller._rows) {
			if (element.scope().controller._rows.length > 0 && 
				!~element.scope().controller._rows.indexOf(widget.currentRow) &&
				!~element.scope().controller._rows.indexOf(previousCurrentRow)) {
				widget.currentRow = element.scope().controller._rows[0];
			}
		}
		if (widget.computeDescriptionOrder) widget.computeDescriptionOrder();
	};

	var pressDownKey = function (widget, dataSource) {
		var previousCurrentRow = widget.currentRow;
		var index = dataSource.indexOf(widget.currentRow);
		if (Util.isEmptyObject(widget.currentRow) || index == -1) {
			widget.currentRow = dataSource[0];
		} else if (index != dataSource.length - 1) {
			var isClosed, row, found = false;
			do {
				index += 1;
				row = dataSource[index];
				isClosed = row.__groupHeaders && row.__groupHeaders.some(function(header) {
					return header.__groupOpened === false;
				});
				found = !isClosed;
			} while (!found && index < dataSource.length - 1);
			if (found) {
				widget.currentRow = dataSource[index];
			}
		}

		var element = angular.element($('.body').last());
		if (element.scope() && element.scope().controller && element.scope().controller._rows) {
			if (element.scope().controller._rows.length > 0 && 
				!~element.scope().controller._rows.indexOf(widget.currentRow) &&
				!~element.scope().controller._rows.indexOf(previousCurrentRow)) {
				widget.currentRow = element.scope().controller._rows[element.scope().controller._rows.length - 1];
			}
		}
		if (widget.computeDescriptionOrder) widget.computeDescriptionOrder();
	};

	var filterVisibleRowsOnTree = function (widget) {
		return widget.plainTreeData.filter(function (row) {
			return Util.isUndefinedOrNull(row._parent) || row._parent._opened;
		});
	};

	var filterSearchVisibleRows = function (widget, data) {
		var fieldsArray = [];
		widget.fields.forEach(function (field) {
			if (field.isVisible && field.aclIsVisible) {
				fieldsArray.push(field.name);
			}
		});
		return data.filter(function (row) {
			var toReturn = false;
			fieldsArray.forEach(function (property) {
				if (row[property] && row[property].toLowerCase().indexOf(widget.searchList.toLowerCase()) != -1) {
					toReturn = true;
				}
			});
			return toReturn;
		});
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register("GridNavigationLibrary", GridNavigationLibrary);
});